package primer_01_static_init;

import java.util.Arrays;

public class Main {

    // Staticki atribut. Inicijalizovan je direktno.
    private static int n = 4;
    private static double f = Math.sin(2);

    // Ne mozemo inicijalizovati sadrzaj niza u jednoj naredbi,
    // pa cemo koristiti staticki inicijalizacioni blok.
    private static int[] binkoef = new int[n + 1];

    // Staticki inicijalizacioni blok se izvrsava jednom,
    // prilikom ucitavanja klase. Unutar statickog
    // inicijalizacionog bloka ne smemo koristiti
    // nestaticke clanove klase, kao i kod statickih metoda
    static {
        // Racunamo niz binomnih koeficijenata
        binkoef[0] = 1;

        for (int i = 1; i < binkoef.length; i++) {
            binkoef[i] = binkoef[i - 1] * (n - i + 1) / i;
        }
    }

    // Alternativni nacin je kreiranje staticke funkcije
    // koju cemo koristiti za inicijalizaciju vrednosti
    // promenljive.
    private static int[] binkoefInit(int n) {
        int[] result = new int[n + 1];

        // Racunamo niz binomnih koeficijenata
        result[0] = 1;

        for (int i = 1; i < result.length; i++) {
            result[i] = result[i - 1] * (n - i + 1) / i;
        }

        return result;
    }

    private static int[] binkoefAlt = binkoefInit(n);

    // Geter za staticku promenljivu binkoef
    public static int[] getBinKoef() {
        return binkoef;
    }

    // Geter za staticku promenljivu n
    public static int getN() {
        return n;
    }

    // Nestaticki deo, dat tek radi ilustracije
    // razlike izmedju konstruktora i statickog
    // bloka za inicijalizaciju

    private String naziv;

    public Main(String _naziv) {
        naziv = _naziv;
    }

    public String toString() {
        return "Ovo je " + naziv;
    }

    // main funkcija za testiranje klase
    public static void main(String[] args) {
        // Staticki clanovi klase su inicijalizovani pre nego
        // sto napravimo instancu iste (cak ni ne moramo nikad
        // da napravimo instancu klase Main)

        System.out.println(Arrays.toString(Main.getBinKoef()));

        Main m = new Main("naziv1");
        System.out.println(m);

    }
}
